package com.simuwang.base.mapper.daq.system;

import com.simuwang.base.pojo.dos.SysConfigDO;
import com.simuwang.base.pojo.dto.query.ConfigPageQuery;
import com.simuwang.base.pojo.vo.SysConfigVO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * FileName: SysConfigMapper
 * Author:   chenjianhua
 * Date:     2024/9/8 12:46
 * Description: ${DESCRIPTION}
 */
@Mapper
public interface SysConfigMapper {
    /**
     * 查询参数配置信息
     *
     * @param config 参数配置信息
     * @return 参数配置信息
     */
    public SysConfigDO selectConfig(SysConfigVO config);

    /**
     * 通过ID查询配置
     *
     * @param configId 参数ID
     * @return 参数配置信息
     */
    public SysConfigDO selectConfigById(@Param("configId") Integer configId);

    /**
     * 查询参数配置列表
     *
     * @param query 参数配置信息
     * @return 参数配置集合
     */
    public List<SysConfigDO> selectConfigList(ConfigPageQuery query);

    /**
     * 根据键名查询参数配置信息
     *
     * @param configKey 参数键名
     * @return 参数配置信息
     */
    public SysConfigDO checkConfigKeyUnique(String configKey);

    /**
     * 新增参数配置
     *
     * @param config 参数配置信息
     * @return 结果
     */
    public int insertConfig(SysConfigDO config);

    /**
     * 修改参数配置
     *
     * @param config 参数配置信息
     * @return 结果
     */
    public int updateConfig(SysConfigDO config);

    /**
     * 删除参数配置
     *
     * @param configId 参数ID
     * @return 结果
     */
    public int deleteConfigById(@Param("configId") Integer configId);

    /**
     * 批量删除参数信息
     *
     * @param configIds 需要删除的参数ID
     * @return 结果
     */
    public int deleteConfigByIds(@Param("configIds") List<Integer> configIds,@Param("userId") Integer userId);


    String selectConfigByKey(@Param("configKey") String configKey);

    long countConfigList(ConfigPageQuery query);

    void updateConfigByKey(SysConfigDO sysConfigDO);
}
