package com.simuwang.daq.components.report.parser.excel;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import com.simuwang.base.common.exception.ReportParseException;
import com.simuwang.base.mapper.daq.EmailFieldMappingMapper;
import com.simuwang.base.pojo.dto.report.*;
import com.simuwang.daq.components.report.parser.ReportParserConstant;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Component(ReportParserConstant.PARSER_EXCEL_MONTHLY)
public class ExcelMonthlyReportParser extends AbstractExcelReportParser<MonthlyReportData> {
    public ExcelMonthlyReportParser(EmailFieldMappingMapper fieldMappingMapper) {
        super(fieldMappingMapper);
    }

    @Override
    public String getParser() {
        return ReportParserConstant.PARSER_EXCEL_MONTHLY;
    }

    @Override
    protected List<CustomExcelTable> customExcelTables() {
        List<CustomExcelTable> customExcelTables = ListUtil.list(true);
        customExcelTables.add(new CustomExcelTable("fundInfo", "基金概况", 4));
        customExcelTables.add(new CustomExcelTable("netReport", "净值月报", 5, 2));
        customExcelTables.add(new CustomExcelTable("netReport", "级基金净值表", 5, 2));
        return customExcelTables;
    }

    @Override
    protected MonthlyReportData parseExtInfoAndSetData(ReportBaseInfoDTO reportInfo, ReportFundInfoDTO reportFundInfo, List<SimpleTable> tables) {
        MonthlyReportData reportData = new MonthlyReportData(reportInfo, reportFundInfo);
        // 取分级基金和当前母基金
        List<SimpleTable> netNavTables = tables.stream().filter(e -> "netReport".equals(e.getTableKey())).collect(Collectors.toList());
        // 母基金和分级基金的净值
        List<ReportNetReportDTO> dtos = this.buildLevelDto(reportInfo.getFileId(), netNavTables,
                ReportNetReportDTO.class, t -> {
                    Map<String, Object> extInfoMap = MapUtil.newHashMap(16);
                    for (int i = 0; i < t.getColCount(); i++) {
                        String key = t.getCell(0, i);
                        String value = t.getCell(1, i);
                        extInfoMap.put(key, value);
                    }
                    return extInfoMap;
                });
        reportData.setNetReport(dtos);
        return reportData;
    }

    @Override
    protected ReportFundInfoDTO buildFundInfo(ReportParserParams params, List<SimpleTable> tables) {
        SimpleTable fundInfoTable = tables.stream().filter(e -> "fundInfo".equals(e.getTableKey())).findFirst().orElse(null);
        if (fundInfoTable == null) {
            throw new ReportParseException(ReportParseStatus.PARSE_FUND_INFO_FAIL, params.getFilename());
        }
        Map<String, Object> baseInfoMap = MapUtil.newHashMap(32);
        for (int i = 0; i < fundInfoTable.getData().size(); i++) {
            @SuppressWarnings("all")
            List<String> row = fundInfoTable.getData().get(i);
            for (int j = 0; j < 2; j++) {
                baseInfoMap.put(row.get(j * 2), row.get(j * 2 + 1));
            }
        }
        ReportFundInfoDTO dto = new ReportFundInfoDTO(params.getFileId());
        this.buildInfo(baseInfoMap, dto);
        return dto;
    }

    @Override
    protected void cleaningReportData(MonthlyReportData reportData) {
        // todo 数据清理
    }
}
