package com.simuwang.manage.service.impl;

import com.simuwang.base.common.support.MybatisPage;
import com.simuwang.base.mapper.daq.CompanyEmailConfigMapper;
import com.simuwang.base.mapper.daq.CompanyEmailSendHistoryMapper;
import com.simuwang.base.pojo.dos.CompanyEmailConfigDO;
import com.simuwang.base.pojo.dos.CompanyEmailSendHistoryDO;
import com.simuwang.base.pojo.dto.query.CompanyEmailHistoryPageQuery;
import com.simuwang.base.pojo.dto.query.CompanyEmailPageQuery;
import com.simuwang.base.pojo.vo.CompanyEmailConfigVO;
import com.simuwang.base.pojo.vo.CompanyEmailSendHistoryVO;
import com.simuwang.base.pojo.vo.IdVO;
import com.simuwang.manage.service.CompanyEmailSendHistoryService;
import com.simuwang.shiro.utils.UserUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * FileName: CompanyEmailSendHistoryServiceImpl
 * Author:   chenjianhua
 * Date:     2024/9/9 10:06
 * Description: ${DESCRIPTION}
 */
@Service
public class CompanyEmailSendHistoryServiceImpl implements CompanyEmailSendHistoryService {
    @Autowired
    private CompanyEmailSendHistoryMapper companyEmailSendHistoryMapper;

    @Autowired
    private CompanyEmailConfigMapper companyEmailConfigMapper;
    @Override
    public MybatisPage<CompanyEmailSendHistoryVO> searchCompanyEmail(CompanyEmailPageQuery companyEmailPageQuery) {
        List<CompanyEmailSendHistoryDO> dataList = companyEmailSendHistoryMapper.searchCompanyEmailList(companyEmailPageQuery);
        List<CompanyEmailSendHistoryVO> voList = dataList.stream().map(CompanyEmailSendHistoryDO::toVo).collect(Collectors.toList());
        long total = companyEmailSendHistoryMapper.countCompanyEmailList(companyEmailPageQuery);
        return MybatisPage.of(total, voList);
    }

    @Override
    public MybatisPage<CompanyEmailSendHistoryVO> searchEmailHistory(CompanyEmailHistoryPageQuery companyEmailHistoryPageQuery) {
        List<CompanyEmailSendHistoryDO> dataList = companyEmailSendHistoryMapper.searchEmailHistory(companyEmailHistoryPageQuery);
        List<CompanyEmailSendHistoryVO> voList = dataList.stream().map(CompanyEmailSendHistoryDO::toVo).collect(Collectors.toList());
        long total = companyEmailSendHistoryMapper.countCompanyEmailhistory(companyEmailHistoryPageQuery);
        return MybatisPage.of(total, voList);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteEmail(List<Integer> idList) {
        Integer userId = UserUtils.getLoginUser().getUserId();
        for(Integer id : idList){
            CompanyEmailConfigDO configDO = companyEmailConfigMapper.selectCompanyEmailConfigById(id);
            //删除配置
            companyEmailConfigMapper.deleteCompanyEmailConfig(configDO.getId(),userId);
            //删除历史
            List<String> emailList = new ArrayList<>();
            emailList.add(configDO.getEmail());
            if(emailList.size() > 0){
                companyEmailSendHistoryMapper.deleteEmailHistory(emailList);
            }
        }
    }

    @Override
    public void deleteEmailHistory(List<Integer> idList) {
        Integer userId = UserUtils.getLoginUser().getUserId();
        companyEmailSendHistoryMapper.deleteEmailHistoryByIds(idList,userId);
    }

    @Override
    public CompanyEmailConfigVO searchEmailConfig(IdVO idVO) {
        CompanyEmailConfigDO configDO = companyEmailConfigMapper.selectCompanyEmailConfigById(idVO.getId());
        CompanyEmailConfigVO configVO = configDO.toVO();
        return configVO;
    }
}
