package com.simuwang.manage.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.simuwang.base.common.util.DateUtils;
import com.simuwang.base.common.util.StringUtil;
import com.simuwang.base.mapper.daq.EmailFundNavMapper;
import com.simuwang.base.mapper.daq.NavMapper;
import com.simuwang.base.pojo.dos.CompanyEmailSendHistoryDO;
import com.simuwang.base.pojo.dos.EmailFundNavDO;
import com.simuwang.base.pojo.dos.NavDO;
import com.simuwang.daq.service.EmailParseService;
import com.simuwang.manage.service.EmailFundNavService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * FileName: EmailFileNavServiceImpl
 * Author:   chenjianhua
 * Date:     2024/9/15 11:25
 * Description: ${DESCRIPTION}
 */
@Service
public class EmailFundNavServiceImpl implements EmailFundNavService {

    @Autowired
    private EmailFundNavMapper emailFundNavMapper;

    @Autowired
    private EmailParseService emailParseService;

    @Autowired
    private NavMapper navMapper;

    @Override
    public void reparseFileNav(String sourceFundName,String registerNumber, String targetFundId,String sourceFundId) {
        List<EmailFundNavDO> fundNavDOList = emailFundNavMapper.selectNotMappingNav(sourceFundName,registerNumber);
        if(fundNavDOList.size() > 0){
            //修改采集库的数据
            for(EmailFundNavDO fundNavDO : fundNavDOList){
                fundNavDO.setFundId(targetFundId);
                fundNavDO.setExceptionStatus(1);
                fundNavDO.setIsStored(1);
                fundNavDO.setUpdateTime(new Date());
            }
            emailFundNavMapper.batchUpdate(fundNavDOList);
            //净值入库
            List<NavDO> navDOList = fundNavDOList.stream().filter(e -> StrUtil.isNotBlank(e.getFundId()))
                    .map(e -> BeanUtil.copyProperties(e, NavDO.class)).collect(Collectors.toList());
            navDOList.forEach(e -> e.setUpdateTime(DateUtils.getNowDate()));
            emailParseService.saveNavDo(navDOList,null);
            //还原历史基金的净值数据
            if(StringUtil.isNotEmpty(sourceFundId)){
                List<String> priceDateList = new ArrayList<>();
                for(EmailFundNavDO fundNavDO : fundNavDOList){
                    priceDateList.add(DateUtils.format(fundNavDO.getPriceDate(),DateUtils.YYYY_MM_DD));
                }
                //将匹配错的数据删除
                navMapper.batchDeleteNav(sourceFundId,priceDateList);
                //从解析库把原来的净值数据重新写入到sourceFundId对应的基金上
                List<EmailFundNavDO> sourceNavList = emailFundNavMapper.selectNavByFundId(sourceFundId);
                List<NavDO> sourcenavDOList = sourceNavList.stream().filter(e -> StrUtil.isNotBlank(e.getFundId()))
                        .map(e -> BeanUtil.copyProperties(e, NavDO.class)).collect(Collectors.toList());
                sourcenavDOList.forEach(e -> e.setUpdateTime(DateUtils.getNowDate()));
                emailParseService.saveNavDo(sourcenavDOList,null);
            }
        }
    }

    @Override
    public Integer getNavAmplitudeErrorCount() {
        return emailFundNavMapper.getNavAmplitudeErrorCount();
    }

    @Override
    public List<EmailFundNavDO> getNavAmplitudeErrorList() {
        return emailFundNavMapper.getNavAmplitudeErrorList();
    }

    @Override
    public void update(EmailFundNavDO fundNavDO) {
        emailFundNavMapper.update(fundNavDO);
    }

    @Override
    public Map<String, Long> selectNavAmplitudeErrorMaxMinId() {
        return emailFundNavMapper.selectNavAmplitudeErrorMaxMinId();
    }

    @Override
    public List<EmailFundNavDO> getNavAmplitudeErrorListById(long startIdx, long endIdx) {
        return emailFundNavMapper.getNavAmplitudeErrorListById(startIdx,endIdx);
    }
}
